/***************************************************************************************
 *	File :	  ixml.h
 *	Company : PDADevelopers.com
 *	Author :  Diego Belfer 
 *	Last Revision : 28/09/2003 (dd/mm/yyyy)
 *	Comments : Interface definition of PDAXML library
 ***************************************************************************************/

#ifndef __INC_IXML
#define __INC_IXML

#include "types.h"

//Error Constants
#define ERR_CLOSE_TAG  1
#define ERR_INV_CHAR   2
#define ERR_INV_SINTAX 3
#define ERR_ELEM_NCLOSED 4
#define ERR_ELEM_NREC	5


//nodeType constants
#define NODE_ELEMENT			1
#define NODE_ATTRIBUTE			2
#define NODE_TEXT				3
#define NODE_CDATA_SECTION 		4
#define NODE_PROCESSING_INSTRUCTION	7
#define NODE_COMMENT			8
#define NODE_DOCUMENT			9
#define NODE_DOCUMENT_TYPE 		10

#ifdef __cplusplus
	struct _XMLNode3;
	struct _XMLParser;
#else
	typedef struct XMLN _XMLNode3;
	typedef struct XMLP _XMLParser;
#endif

typedef _XMLNode3*  XMLNode;
typedef _XMLParser* HXML;

HXML 		xml_create();
void 		xml_release(HXML h);

char* 		xml_getDocument(HXML h);
BOOL		xml_processDocument(HXML h, char *XMLdocument);

XMLNode		xml_getRoot(HXML h);
XMLNode		xml_parentNode(XMLNode node);
XMLNode		xml_firstChild(XMLNode node);
XMLNode		xml_lastChild(XMLNode node);
XMLNode		xml_nextSibling(XMLNode node);
XMLNode		xml_previousSibling(XMLNode node);
XMLNode		xml_firstAttribute(XMLNode node);
XMLNode		xml_lastAttribute(XMLNode node);

uint		xml_attributeCount(XMLNode node);
uint		xml_childCount(XMLNode node);
BOOL		xml_hasChildNodes(XMLNode node);

XMLNode		xml_getElementsByTagName(XMLNode node, const char*name);
XMLNode		xml_getElementsByTagName_next(XMLNode node, const char*name);

XMLNode		xml_getAttributeNode(XMLNode node, const char*name);
char*		xml_getAttribute(XMLNode node, const char*name, char *buffer, uint size);

uint		xml_nodeType(XMLNode node);
char*		xml_nodeName(XMLNode node, char *buffer, uint size);
char*		xml_nodeValue(XMLNode node, char *buffer, uint size);
char*		xml_text(XMLNode node, char *buffer, uint size);


uint		xml_getErrorID(HXML h);
char*		xml_getErrorMsg(HXML h);
uint		xml_getErrorPos(HXML h);

//Editing Functions

XMLNode		xml_createRoot(HXML h); 
XMLNode		xml_createDocumentFragment(HXML h);
XMLNode		xml_createElement(HXML h, const char *tagName);
#ifdef __cplusplus
XMLNode		xml_createAttribute(HXML h, const char *name, const char *value = "");
XMLNode		xml_createCDATASection(HXML h, const char* data = "");
XMLNode		xml_createTextNode(HXML h, const char *text = "");
XMLNode		xml_createComment(HXML h, const char *comment = "");
XMLNode		xml_createProcessingInstruction(HXML h, const char *target = "xml", const char *data="version=\"1.0\"");
#else
XMLNode		xml_createAttribute(HXML h, const char *name, const char *value); //value = ""
XMLNode		xml_createCDATASection(HXML h, const char* data);// data = ""
XMLNode		xml_createTextNode(HXML h, const char *text); // text = ""
XMLNode		xml_createComment(HXML h, const char *comment); // comment = ""
XMLNode		xml_createProcessingInstruction(HXML h, const char *target, const char *data); //target = "xml", data = "version=\"1.0\""
#endif


XMLNode		xml_appendChild(XMLNode parent, XMLNode node);
XMLNode		xml_appendAttribute(XMLNode parent, XMLNode attr);
XMLNode		xml_cloneNode(XMLNode node, BOOL deep);

XMLNode		xml_insertBefore(XMLNode newNode, XMLNode before);
XMLNode		xml_remove(XMLNode node);	
XMLNode		xml_replace(XMLNode newNode, XMLNode oldNode);

BOOL		xml_delete(XMLNode node);	
BOOL		xml_deleteFreeNodes(HXML h);
//End editing Functions

uint		xml_toXML(HXML h, char *buffer, uint size);

#endif