/***************************************************************************************
 *	File :	  Starter.cpp
 *	Company : PDADevelopers.com
 *	Author :  Diego Belfer 
 *	Last Revision : 28/09/2003 (dd/mm/yyyy)
 *	Comments : Main file of Palm Samples
 ***************************************************************************************/


#include <PalmOS.h>
#include "StarterRsc.h"
#include "common.h"
#include "samples.h"


#define USE_WRAPPER 1  //Set to 1 to use CPP wrapper


/***********************************************************************
 *
 *   Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *   Internal Structures
 *
 ***********************************************************************/
typedef struct 
	{
	UInt8 replaceme;
	} StarterPreferenceType;

typedef struct 
	{
	UInt8 replaceme;
	} StarterAppInfoType;

typedef StarterAppInfoType* StarterAppInfoPtr;


/***********************************************************************
 *
 *   Global variables
 *
 ***********************************************************************/
//static Boolean HideSecretRecords;


/***********************************************************************
 *
 *   Internal Constants
 *
 ***********************************************************************/
#define appFileCreator				 'STRT'
#define appVersionNum              0x01
#define appPrefID                  0x00
#define appPrefVersionNum          0x01

// Define the minimum OS version we support (2.0 for now).
#define ourMinVersion	sysMakeROMVersion(2,0,0,sysROMStageRelease,0)


/***********************************************************************
 *
 *   Internal Functions
 *
 ***********************************************************************/
char printBuffer[2048];

void oprint(char *text) {
	StrCat(printBuffer, text);
}

void oprintln(char *text) {
	StrCat(printBuffer, text);
	StrCat(printBuffer, "\n");

}
void oprint(uint value) {
	char bufferNum[20];
	sprintf(bufferNum, "%u", value);
	oprint(bufferNum);
}

void oprintln(uint value) {
	char bufferNum[20];
	sprintf(bufferNum, "%u", value);
	oprintln(bufferNum);
}

/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
			FrmAlert (RomIncompatibleAlert);
		
			// Palm OS 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion < ourMinVersion)
				{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
				}
			}
		
		return sysErrRomIncompatible;
		}

	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    void *
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void * GetObjectPtr(UInt16 objectID)
{
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}


/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr /*frmP*/)
{
}


void doScrollChanged(unsigned short newValue, unsigned short oldValue) {
	FieldType* fld = (FieldType*)GetObjectPtr(MainTextField);
	
	if (newValue > oldValue)
		FldScrollField(fld, newValue-oldValue, winDown);
	else
		FldScrollField(fld, oldValue-newValue, winUp);
}


void doPageUpDown(BOOL up) {
	short scrollPos, minValue, maxValue, pageSize, newPos;

	FieldType* fld = (FieldType*)GetObjectPtr(MainTextField);
	ScrollBarType* bar =  (ScrollBarType*)GetObjectPtr(MainBarScrollBar);
	SclGetScrollBar (bar, &scrollPos, &minValue, &maxValue, &pageSize);
	
	if (!up) {
		newPos = scrollPos + pageSize;
		if (newPos > maxValue) newPos = maxValue;
	}
	else {
		newPos = scrollPos - pageSize;
		if (newPos < minValue) newPos = minValue;
	}
	
	doScrollChanged(newPos, scrollPos);
	SclSetScrollBar (bar, newPos, 0, maxValue, pageSize);	
}

void doFieldChanged() {
	unsigned short textHeight, fieldHeight, scrollPos, maxValue;
	FieldType* fld = (FieldType*)GetObjectPtr(MainTextField);
	
	FldGetScrollValues (fld, &scrollPos, &textHeight, &fieldHeight);
	if (textHeight > fieldHeight)
		maxValue = textHeight - fieldHeight;
	else if (scrollPos)
			maxValue = scrollPos;
		else
			maxValue = 0;
			
	ScrollBarType* bar =  (ScrollBarType*)GetObjectPtr(MainBarScrollBar);
	SclSetScrollBar (bar, scrollPos, 0, maxValue, fieldHeight-1);
}

void setTextToField(unsigned long fldID, char* text) {
	
	FieldType *fld = (FieldType*)GetObjectPtr(fldID);
	MemHandle memH = FldGetTextHandle(fld);
	char* dstP;

	if (memH) {
		FldSetTextHandle(fld, NULL);
		MemHandleResize(memH, StrLen(text)+1);
	}
	else {
		memH = MemHandleNew(StrLen(text)+1);
	}
	dstP = (char*)MemHandleLock(memH);
	MemMove(dstP, text, StrLen(text)+1);
	
	MemHandleUnlock(memH);
	FldSetTextHandle(fld,memH);
	FldDrawField(fld);
	doFieldChanged();
}

void doGo() {
	char* source = " <?xml version=\"1.0\" standalone=\"yes\" ?> <ContractsDS>  <Contracts>  <WO_NUM>2515170</WO_NUM>  <f0>21</f0>  <f1>876</f1>  <f2>06/28/2004</f2>  <f3>06/27/2005</f3>  <f4>951105258 - ST VINCENT HOSPITAL GREEN BAY ,WI</f4>  <f5>1 HOUR</f5>  <f6>4 HOURS</f6>  <f7>Included</f7>  <f8>Included</f8>  <f9>08:00 - 17:00 Monday - Friday</f9>  <f10>Included</f10> </Contracts> </ContractsDS> ";


	printBuffer[0] = 0;
	sampleBuffer[0] = 0;
#if USE_WRAPPER
	editCPP();
	parseCPP(sampleBuffer);
#else
	editAPI();
	parseAPI(sampleBuffer);
#endif
	setTextToField(MainTextField, printBuffer);
	
}

void doHelp() {
	char *hlpText = "This sample generates a XML document using the editing capabilities of the library. After the document is generated, it is parsed to obtain the values that fill the sample forms.\nYou can see the code used to build the xml document in editapi.cpp and editcpp.cpp. The sample code that parses the xml document is in parseapi.cpp and parsecpp.cpp.\nYou can choose between use the CPP wrapper class or direct functions call to the library setting/unsetting the USE_WRAPPER define at the begining of the file Starter.cpp";

	setTextToField(MainTextField, hlpText);
}


/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;

	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
   Boolean handled = false;
   FormPtr frmP;

	switch (eventP->eType) {

		case penDownEvent:
			return SclHandleEvent((ScrollBarType*)GetObjectPtr(MainBarScrollBar),  eventP);

		case penUpEvent:
			return SclHandleEvent((ScrollBarType*)GetObjectPtr(MainBarScrollBar),  eventP);

		
		case menuEvent:
			return MainFormDoCommand(eventP->data.menu.itemID);

		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			MainFormInit( frmP);
			FrmDrawForm ( frmP);
			handled = true;
			break;

		case ctlSelectEvent:
			if (eventP->data.ctlSelect.controlID == MainHelpButton)
				doHelp();
			else
				doGo();

			return true;
		
		case sclRepeatEvent:
			if (eventP->data.sclRepeat.newValue !=eventP->data.sclRepeat.value)
				doScrollChanged(eventP->data.sclRepeat.newValue, eventP->data.sclRepeat.value);
			break;
		
		case keyDownEvent:
			if (eventP->data.keyDown.chr == 11) 
				doPageUpDown(TRUE);
			else
				if (eventP->data.keyDown.chr == 12) 
					doPageUpDown(FALSE);
			break;
		
		case fldChangedEvent:
			doFieldChanged();
			break;
		default:
			break;
		
	}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
		{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
			{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;

			default:
//				ErrFatalDisplay("Invalid Form Load Event");
				break;

			}
		return true;
		}
	
	return false;
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
    StarterPreferenceType prefs;
    UInt16 prefsSize;

	// Read the saved preferences / saved-state information.
	prefsSize = sizeof(StarterPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, &prefs, &prefsSize, true) != 
		noPreferenceFound)
		{
		}
	
   return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
   StarterPreferenceType prefs;

	// Write the saved preferences / saved-state information.  This data 
	// will be backed up during a HotSync.
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof (prefs), true);
		
	// Close all the open forms.
	FrmCloseAllForms ();
}


/***********************************************************************
 *
 * FUNCTION:    StarterPalmMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static UInt32 StarterPalmMain(UInt16 cmd, MemPtr /*cmdPBP*/, UInt16 launchFlags)
{
	Err error;

	error = RomVersionCompatible (ourMinVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
		{
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(MainForm);
			AppEventLoop();
			AppStop();
			break;

		default:
			break;

		}
	
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    return StarterPalmMain(cmd, cmdPBP, launchFlags);
}

